VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Sheet9"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
Option Explicit

'*** WORKSHEET EVENTS ***

Private Sub Worksheet_Change(ByVal Target As Range)
'activated by change in either custom picklist cell

With ActiveSheet
If Target.Address = [customconstantpick].Address Or _
    Target.Address = [customstandardpick].Address Then
'1. show scenario of the same name as the target cell, which sets RGB cells
    .Scenarios(Target.Value).Show
'2. set the colorrange fill to the colornum value
    .[ColorRange].Interior.Color = [ColorNum]
End If
End With
End Sub

Private Sub Worksheet_SelectionChange(ByVal Target As Range)
'activated by selection change within the colortabs range

'1. hide all columns and controls
If Application.Intersect(Target, [colortabs]) Is Nothing Or Target.Cells.Count > 1 Then Exit Sub
Range([ColorOverviewCols], [ColorIndexCols]).Columns.Hidden = True
ToggleButtonTheme.Visible = False
ToggleButtonStandard.Visible = False
ScrollBarRed.Visible = False
ScrollBarGreen.Visible = False
ScrollBarBlue.Visible = False
CommandButtonRandom.Visible = False
ToggleButtonIndex.Visible = False

'2. reset colortabs range, then formats the selected tab cell
[colortabs].Interior.TintAndShade = 0.4
[colortabs].Font.Bold = False
Target.Interior.TintAndShade = 0.8
Target.Font.Bold = True

'3. use the target cell value to unhide the selected column group and controls
Select Case Target.Value
Case "Overview"
    With [ColorOverviewCols]: .Columns.Hidden = False: End With
Case "Dialog"
    With [ColorDialogCols]: .Columns.Hidden = False: End With
Case "Theme"
    With [ColorThemeCols]: .Columns.Hidden = False: End With
    ToggleButtonTheme.Visible = True
Case "Standard"
    With [ColorStandardCols]: .Columns.Hidden = False: End With
    ToggleButtonStandard.Visible = True
Case "Custom"
    With [ColorCustomCols]: .Columns.Hidden = False: End With
    ScrollBarRed.Visible = True
    ScrollBarGreen.Visible = True
    ScrollBarBlue.Visible = True
Case "Random"
    With [ColorRandomCols]: .Columns.Hidden = False: End With
    CommandButtonRandom.Visible = True
Case "Index"
    With [ColorIndexCols]: .Columns.Hidden = False: End With
    ToggleButtonIndex.Visible = True
Case "All"
    Range([ColorOverviewCols], [ColorIndexCols]).Columns.Hidden = False
    ToggleButtonTheme.Visible = [showcontrols]
    ToggleButtonStandard.Visible = [showcontrols]
    ScrollBarRed.Visible = [showcontrols]
    ScrollBarGreen.Visible = [showcontrols]
    ScrollBarBlue.Visible = [showcontrols]
    CommandButtonRandom.Visible = [showcontrols]
    ToggleButtonIndex.Visible = [showcontrols]
End Select
Columns("A:A").ColumnWidth = 14

End Sub

'*** USER CONTROLS ***

'Theme toggle button
Private Sub ToggleButtonTheme_Click()
Call SetColorTheme
End Sub

'Standard toggle button
Private Sub ToggleButtonStandard_Click()
Call SetColorStandard
End Sub

'Custom Red scroll bar
Private Sub ScrollBarRed_Change()
Call SetColorCustom
End Sub

'Custom Green scroll bar
Private Sub ScrollBarGreen_Change()
Call SetColorCustom
End Sub

'Custom Blue scroll bar
Private Sub ScrollBarBlue_Change()
Call SetColorCustom
End Sub

'Random command button
Private Sub CommandButtonRandom_Click()
Call SetColorRandom
End Sub

'Index toggle button
Private Sub ToggleButtonIndex_Click()
Call SetColorIndex
End Sub

'*** CONTROL SUBROUTINES ***

'set/reset Theme color grid and table
Public Sub SetColorTheme()
Dim Tbl As ListObject, dbr As Range, n As Integer
Dim tc As Long, ts As Long, fc As Long
On Error Resume Next
Set Tbl = ActiveSheet.ListObjects("ColorTheme")
tc = Tbl.ListColumns("TC").index
ts = Tbl.ListColumns("TS").index
fc = Tbl.ListColumns("font").index
Set dbr = Tbl.DataBodyRange
With [colorthemegrid]
    For n = 1 To .Cells.Count
        With Application.Union(.Item(n), Tbl.ListRows(n).Range)
            .Interior.ThemeColor = IIf([settheme], dbr(n, tc), 1)
            .Interior.TintAndShade = IIf([settheme], dbr(n, ts), 0)
            .Font.ThemeColor = IIf([settheme], dbr(n, fc), 2)
            .Font.TintAndShade = 0
        End With
    Next n
End With
End Sub

'set/reset Standard color grid and table
Public Sub SetColorStandard()
Dim c As Range, dbr As Range
Dim Tbl As ListObject, clr As Long, n As Integer
Dim ic As Long, fc As Long
On Error Resume Next
Set Tbl = ActiveSheet.ListObjects("ColorStandard")
Set dbr = Tbl.DataBodyRange
ic = Tbl.ListColumns("decimal").index
fc = Tbl.ListColumns("font").index
For Each c In [colorstandardgrid].Cells
    n = c.Value
    If n > 0 Then
        With Application.Union(c, Tbl.ListRows(n).Range)
            .Interior.Color = IIf([setstandard], _
                dbr(n, ic), vbWhite)
            .Font.Color = IIf([setstandard] And _
                dbr(n, fc) = 1, vbWhite, vbBlack)
        End With
    End If
Next
End Sub

'calculate Custom color decimal and hex values
Sub SetColorCustom()
Dim DecVal As Long, HexVal As String
On Error Resume Next
With ActiveSheet
    DecVal = RGB([Red], [Green], [Blue])
    HexVal = _
        Right("0" & Hex([Red]), 2) & _
        Right("0" & Hex([Green]), 2) & _
        Right("0" & Hex([Blue]), 2)
    .[ColorRange].Interior.Color = DecVal
    .[ColorNum] = DecVal
    .[ColorHex] = HexVal
End With
End Sub

'refresh Random color grid
Public Sub SetColorRandom()
Dim c As Range, r, g, b As Integer
For Each c In [ColorRandomGrid]
With WorksheetFunction
    r = .RandBetween(0, 255)
    g = .RandBetween(0, 255)
    b = .RandBetween(0, 255)
End With
c.Interior.Color = RGB(r, g, b)
Next
End Sub

'set/reset Index color grid and table
Public Sub SetColorIndex()
Dim c As Range, dbr As Range
Dim Tbl As ListObject, n As Integer, fc As Long
On Error Resume Next
Set Tbl = ActiveSheet.ListObjects("ColorIndex")
Set dbr = Tbl.DataBodyRange
fc = Tbl.ListColumns("Font").index
For Each c In [ColorIndexGrid].Cells
    n = c.Value
    With Application.Union(c, Tbl.ListRows(n).Range)
        .Interior.ColorIndex = IIf([setindex], n, 0)
        .Font.ColorIndex = IIf([setindex], dbr(n, fc), 1)
    End With
Next
End Sub

Sub ShowColorDecimalOrder()
'proof of backward decimal significance

MsgBox "RGB() returns decimal in significance order BGR:" & vbCr & _
        "RGB(0,0,1) = " & RGB(0, 0, 1) & " = Hex " & Hex(RGB(0, 0, 1)) & vbCr & _
        "RGB(0,1,0) = " & RGB(0, 1, 0) & " = Hex " & Hex(RGB(0, 1, 0)) & vbCr & _
        "RGB(1,0,0) = " & RGB(1, 0, 0) & " = Hex " & Hex(RGB(1, 0, 0))

End Sub

