VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Sheet3"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
Option Explicit

Sub ShiftDates()
'use this to shift dates in a sheet

Dim cD, cSet As Range         'cD=date cell, cSet=set of all cells selected for shifting
Dim dLo, lbhLo As Long        'dLo=oldest date, lbhLo=oldest in lookback horizon
Dim dC, pC, tC, fC As Long    'dC, pC, tC, fC count of dates (all,past,today,future)
Dim dS, lbH As Long           'dS=date shift, lbH=lookback horizon
Dim dcS, msg As String        'dcS=date count string; msg=text placeholder
Const lbDef As Long = -365    'default start (lookback), -past, +future

'get the count of dates past, today, and future, and earliest date
On Error GoTo out:
With WorksheetFunction
  For Each cD In ActiveSheet.UsedRange.Cells
    If IsDate(cD) And Not .IsFormula(cD) And Not .IsText(cD) Then
    Select Case Date - cD.Value
      Case Is > 0:  pC = pC + 1 'past count
      Case Is = 0:  tC = tC + 1 'today count
      Case Is < 0:  fC = fC + 1 'future count
    End Select
    
    'find the oldest date
    If dLo = 0 Then dLo = cD.Value Else dLo = .Min(cD.Value, dLo)

    'collect the date cells into cSet and select
    If cSet Is Nothing Then Set cSet = cD Else Set cSet = Union(cSet, cD)
    End If
  Next
  cSet.Select

'sum counts
dC = fC + tC + pC

'exit if no date constants found
If dC = 0 Then MsgBox "No shiftable dates in this sheet", vbInformation, "Shift Dates": Exit Sub

'set up the input box message
msg = "dates" & vbTab & "count" & vbTab & "earliest" & vbCr & _
      "---------" & vbTab & "-----" & vbTab & "-------------" & vbCr & _
      "past" & vbTab & pC & vbTab & IIf(CDate(dLo) < Date, CDate(dLo), "") & vbCr & _
      "today" & vbTab & tC & vbTab & IIf(CDate(dLo) = Date, CDate(dLo), "") & vbCr & _
      "future" & vbTab & fC & vbTab & IIf(CDate(dLo) > Date, CDate(dLo), "") & vbCr & _
      "---------" & vbTab & "-----" & vbCr & _
      "total" & vbTab & dC & vbCr & vbCr & _
      "Enter 1, 2, 3 or 4:" & vbCr & _
      "1: shift dates collectively" & vbCr & _
      "2: shift dates collectively (with lookback horizon)" & vbCr & _
      "3: shift dates individually" & vbCr & _
      "4: shift dates individually (with lookback horizon)"

'process the user's selection
Select Case InputBox(msg, "Shift Dates")
  
  Case 1  '1: shift dates collectively
    
    '1a. input the shift offset (or accept default)
    dS = CLng(Date - dLo) 'define the default offset
    dS = CLng(InputBox("Enter the offset to shift ALL " & dC & _
    " DATES (default = today forward)", "Shift Dates", dS))
    
    '1b. shift the dates
    For Each cD In cSet
      cD.Value = cD.Value + dS
    Next
  
  Case 2  '2: shift dates collectively (with lookback horizon)
    dC = 0: Set cSet = Nothing 'initialize variables
    
    '2a. input the lookback horizon days, with default
    lbH = InputBox("Set lookback horizon for date shift (-past, +future):", "Shift Dates", lbDef)
    
    '2b. find the count and earliest date in lbH
    For Each cD In ActiveSheet.UsedRange.Cells
      If IsDate(cD) And Not .IsFormula(cD) And Not .IsText(cD) And cD >= Date + lbH Then
        If lbhLo = 0 Then lbhLo = cD.Value Else lbhLo = .Min(cD.Value, lbhLo)
        If cSet Is Nothing Then Set cSet = cD Else Set cSet = Union(cSet, cD)
        dC = dC + 1
      End If
    Next
    
    'exit if no date constants found, else select
    If dC = 0 Then MsgBox "No dates in lookback horizon", vbInformation, "Shift Dates": Exit Sub
    cSet.Select
    
    '2c. input the shift offset
    dS = CLng(Date - lbhLo) 'define the default offset to today
    dS = CLng(InputBox("Enter the offset to shift ALL " & dC & " DATES FROM " & Abs(lbH) & _
    " DAYS " & IIf(lbH > 0, "IN FUTURE", "IN PAST") & " ON (default = today forward)", "Shift Dates", dS))
    
    '2d. shift the dates
    For Each cD In cSet
      cD.Value = cD.Value + dS
    Next
  
  Case 3  '3: shift dates individually
    
    'copy cell count to string and reset counter
    dcS = CStr(dC): dC = 0
    
    '3a. for each cell: calc offset, increment counter, activate
    For Each cD In cSet
      dS = CLng(Date - cD)
      dC = dC + 1
      cD.Activate
      
      '3b. input offset and shift date
      cD.Value = cD.Value + InputBox("Enter days to shift " & cD & " (date " & _
      dC & " of " & dcS & ") by:" & vbCr & " default= move to today, " & vbCr & _
      " enter 0 to leave unchanged," & vbCr & " press Esc to exit", "Shift Dates", dS)
    Next
  
  Case 4  '4: shift dates individually (with lookback horizon)
    
    'copy cell count to string, reset counter and cSet
    dcS = CStr(dC): dC = 0: Set cSet = Nothing
    
    '4a. input the lookback horizon days
    lbH = InputBox("Set lookback horizon for date shift (-past, +future):", "Shift Dates", lbDef)
    
    '4b. select date cells in lbh one by one
    For Each cD In ActiveSheet.UsedRange.Cells
      If IsDate(cD) And Not .IsFormula(cD) And Not .IsText(cD) And cD >= Date + lbH Then
        If cSet Is Nothing Then Set cSet = cD Else Set cSet = Union(cSet, cD)
        dC = dC + 1
      End If
    Next
    
    'exit if no date constants found, else select
    If dC = 0 Then MsgBox "No dates in lookback horizon", vbInformation, "Shift Dates": Exit Sub
    cSet.Select
    
    '4c. input offset and shift date
    For Each cD In cSet
      dS = CLng(Date - cD)
      dC = dC + 1
      cD.Activate
      
      '4d. input offset and shift date
      cD.Value = cD.Value + InputBox("Enter days to shift " & cD & " (date " & _
      dC & " of " & dcS & ") by:" & vbCr & " default= move to today, " & vbCr & _
      " enter 0 to leave unchanged," & vbCr & " press Esc to exit", "Shift Dates", dS)
    Next
  
  Case Else ' cancel or any other entry exits with message
    MsgBox "To try again, enter option 1, 2, 3 or 4", , "Shift Dates"

End Select
End With

out:
Select Case Err
  Case 6: MsgBox "Sheet contains invalid dates (6:overflow)", vbCritical, "Shift Dates"
  Case 13: MsgBox "Operation canceled (13:type mismatch)", vbInformation, "Shift Dates"
  Case Else: If Err > 0 Then MsgBox "Error " & Err & ": " & Err.Description, vbCritical, "Shift Dates"
End Select
ActiveCell.Select
End Sub

Sub SelectDates()
'for illustration only
Dim cD, cSet As Range

'first, select all cells where IsDate
  For Each cD In ActiveSheet.UsedRange.Cells
    If IsDate(cD) Then
        If cSet Is Nothing Then
            Set cSet = cD
        Else
            Set cSet = Union(cSet, cD)
        End If
    End If
  Next
cSet.Select
MsgBox "Cells are selected where IsDate(cell); includes formula and text cells.", , "Select Dates"

'second, remove cells where .IsFormula or .IsText
Set cSet = Nothing
With WorksheetFunction
  For Each cD In ActiveSheet.UsedRange.Cells
    If IsDate(cD) And Not .IsFormula(cD) And Not .IsText(cD) Then
        If cSet Is Nothing Then
            Set cSet = cD
        Else
            Set cSet = Union(cSet, cD)
        End If
    End If
  Next
End With
cSet.Select
MsgBox "Cells are selected where IsDate(cell) And Not .IsFormula(cell) And Not .IsText(cell); " & _
  "these are the cells that will be selected for shifting.", , "Select Dates"
ActiveCell.Select
End Sub

Sub ToggleNotes()
'toggle display of Notes column group
On Error GoTo out:
'Chr(149) and Chr(150) are < and > arrows,
'replaced by icon, remain hidden as toggle
If [viewnotes] = Chr(149) Then
  [colgrpnotes].EntireColumn.Hidden = True
  [viewnotes] = Chr(150)
Else
  [colgrpnotes].EntireColumn.Hidden = False
  [viewnotes] = Chr(149)
End If
out:
End Sub

Sub ToggleChart()
'toggle display of Chart column group
On Error GoTo out:
'Chr(149) and Chr(150) are < and > arrows,
'replaced by icon, remain hidden as toggle
If [viewchart] = Chr(149) Then
  [colgrpchart].EntireColumn.Hidden = True
  [viewchart] = Chr(150)
Else
  [colgrpchart].EntireColumn.Hidden = False
  [viewchart] = Chr(149)
End If
out:
End Sub


