VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Sheet1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
Const MaxScens As Integer = 99

'*******************************
'* toggle display of chart,    *
'* activated by checkbox       *
'*******************************
Sub ScreenChange()
With ActiveSheet.Shapes.Range(Array("PlanChart"))
    .Visible = Not .Visible
End With
End Sub

'*******************************
'*     Spin Button Controls    *
'*******************************

Private Sub SpinDem_SpinDown()
Call ScenSpin("Dem", False)
End Sub

Private Sub SpinDem_SpinUp()
Call ScenSpin("Dem", True)
End Sub

Private Sub SpinSup_SpinDown()
Call ScenSpin("Sup", False)
End Sub

Private Sub SpinSup_SpinUp()
Call ScenSpin("Sup", True)
End Sub

Private Sub SpinSST_SpinDown()
Call ScenSpin("SST", False)
End Sub

Private Sub SpinSST_SpinUp()
Call ScenSpin("SST", True)
End Sub

Private Sub SpinSTI_SpinDown()
Call ScenSpin("STI", False)
End Sub

Private Sub SpinSTI_SpinUp()
Call ScenSpin("STI", True)
End Sub

'*****************************************
'* generic spin function, called by      *
'* each up and down spin button          *
'*****************************************
Sub ScenSpin(ScenAbbr As String, Up As Boolean)
Dim ScenRoot, ScenName, ScenDesc As String
Dim ScenNum, ScenCount As Integer
On Error GoTo oops:
ScenRoot = Root(ScenAbbr)
With Application
    .ScreenUpdating = False
    With Sheets("Demo")

'count the scenarios with the same name root
        For Each Scen In .Scenarios
            If InStr(Scen.Name, ScenRoot) Then
                ScenCount = ScenCount + 1
            End If
        Next
        
'set the scenario count cell
        With Sheets("Data")
        .Range("ScensCnt" & ScenAbbr) = ScenCount
        
'increment or decrement scenario number cell
        With .Range("ScenThis" & ScenAbbr)
            If Up Then
                If .Value >= ScenCount Then .Value = 1 Else .Value = .Value + 1
            Else
                If .Value = 1 Then .Value = ScenCount Else .Value = .Value - 1
            End If
        ScenName = ScenRoot & " " & .Value
        End With
        End With
'show the selected scenario
        With .Scenarios(ScenName)
            .Show
            ScenDesc = Left(.Comment, WorksheetFunction.Min(31, InStr(.Comment, ";") - 1))
        End With
        
'set the scenario description cell
        Sheets("Data").Range("ScenDesc" & ScenAbbr) = ScenDesc
    End With
    .ScreenUpdating = True
End With
Exit Sub
oops:
MsgBox ("Error " & Err & " " & Error)
End Sub


'*******************************
'*  Create Scenario Buttons    *
'*******************************

Sub CreateDemScenario()
Call ScenCreate("Dem")
End Sub

Sub CreateSupScenario()
Call ScenCreate("Sup")
End Sub

Sub CreateSSTScenario()
Call ScenCreate("SST")
End Sub

Sub CreateSTIScenario()
Call ScenCreate("STI")
End Sub

'***************************************
'*    Create Scenario Routine          *
'*    called by 4 buttons              *
'***************************************

Sub ScenCreate(ScenAbbr As String)
Dim NewCount As Integer
Dim ScenRoot, ScenDesc As String
On Error GoTo oops:
ScenRoot = Root(ScenAbbr)
With Sheets("Demo")
    
'increment the next scenario number and stop if over max
    NewCount = Sheets("Data").Range("ScensCnt" & ScenAbbr) + 1
    If NewCount > MaxScens Then
        MsgBox "Maximum " & MaxScens & " scenarios reached.", , "Add Scenario"
        Exit Sub
    Else
        
'prompt to continue with creation
        If MsgBox("Add new " & UCase(ScenRoot) & _
            " scenario with current values?", vbQuestion + vbOKCancel, _
            "Add Scenario") = vbCancel Then Exit Sub
    End If
    
    With .Range("Random")
        If .Value Then .Value = 1 Else .Value = 0
    End With
    
'prompt to enter scenario description
    ScenDesc = InputBox("Enter a short description " & _
            "for the new scenario" & vbCr & "(max 30 characters):", _
            "Add Scenario")

'add the new scenario
    .Scenarios.add _
        Name:=ScenRoot & " " & NewCount, _
        ChangingCells:=.Range("ScenCells" & ScenAbbr), _
        Comment:=ScenDesc & ";"
    
'set the scenario number, count and description cells
    With Sheets("Data")
        .Range("ScenThis" & ScenAbbr) = NewCount
        .Range("ScensCnt" & ScenAbbr) = NewCount
        .Range("ScenDesc" & ScenAbbr) = ScenDesc
    End With
End With
Exit Sub
oops:
MsgBox ("Error " & Err & " " & Error)
End Sub

'*******************************
'*  Change Scenario Buttons    *
'*******************************

Sub ChangeDemScenario()
Call ScenChange("Dem")
End Sub

Sub ChangeSupScenario()
Call ScenChange("Sup")
End Sub

Sub ChangeSSTScenario()
Call ScenChange("SST")
End Sub

Sub ChangeSTIScenario()
Call ScenChange("STI")
End Sub

'***************************************
'*    Change Scenario Routine          *
'*    called by 4 buttons              *
'***************************************

Sub ScenChange(ScenAbbr As String)
Dim ScenName, ScenDesc, NewDesc As String
On Error GoTo oops:
With Sheets("Demo")
    ScenName = Root(ScenAbbr) & " " & Sheets("Data").Range("ScenThis" & ScenAbbr)

'prompt to continue with change
    If MsgBox("Update scenario " & UCase(ScenName) & " with current values?", _
        vbQuestion + vbOKCancel, "Change Scenario") = vbCancel Then Exit Sub
    
    With .Range("Random")
        If .Value Then .Value = 1 Else .Value = 0
    End With
    
'prompt to change description
    With .Scenarios(ScenName)
        ScenDesc = Left(.Comment, WorksheetFunction.Min(31, InStr(.Comment, ";") - 1))
    End With
    NewDesc = InputBox("Current description: " & _
    ScenDesc & vbCr & "Enter new description, " & _
    "or leave blank to keep current.", "Change Scenario")
    
    If Len(NewDesc) Then ScenDesc = NewDesc
    
'delete and recreate the scenario with current cell values and new description
    .Scenarios(ScenName).Delete
    .Scenarios.add _
        Name:=ScenName, _
        ChangingCells:=.Range("ScenCells" & ScenAbbr), _
        Comment:=ScenDesc & ";"
    
'set scenario description cell
    Sheets("Data").Range("ScenDesc" & ScenAbbr) = ScenDesc
End With
Exit Sub
oops:
MsgBox Err & " " & Error
End Sub


'*******************************
'*  Delete Scenario Buttons    *
'*******************************

Sub DeleteDemScenario()
Call ScenDelete("Dem")
End Sub

Sub DeleteSupScenario()
Call ScenDelete("Sup")
End Sub

Sub DeleteSSTScenario()
Call ScenDelete("SST")
End Sub

Sub DeleteSTIScenario()
Call ScenDelete("STI")
End Sub

'***************************************
'*    Delete Scenario Routine          *
'*    called by 4 buttons              *
'***************************************

Sub ScenDelete(ScenAbbr As String)
Dim ScenRoot, DelName, Prompt, ScenShow, ScenDesc As String
Dim ScenCount, DelNum, ReNum, ShowNum As Integer
On Error GoTo oops:
ScenRoot = Root(ScenAbbr)
With Sheets("Demo")
    DelNum = Sheets("Data").Range("ScenThis" & ScenAbbr)
    
'count the scenarios with the same name root
    For Each Scen In .Scenarios
        If InStr(Scen.Name, ScenRoot) Then ScenCount = ScenCount + 1
    Next
    DelName = ScenRoot & " " & DelNum
    
'prompt to continue with deletion/renumbering
    If DelNum < ScenCount Then Prompt = " and renumber the " & _
            ScenCount - DelNum & " above it"
    If MsgBox("Delete scenario " & UCase(DelName) & Prompt & "?", _
        vbQuestion + vbOKCancel, _
        "Delete Scenario") = vbCancel Then Exit Sub
        
'delete scenario
    .Scenarios(DelName).Delete

'renumber scenarios with higher numbers
    If DelNum < ScenCount Then
    For Each Scen In .Scenarios
        With Scen
            If InStr(.Name, ScenRoot) Then
                ReNum = CInt(Right(.Name, Len(.Name) - Len(ScenRoot) - 1))
                If ReNum > DelNum Then
                    .Show
                    Call ScenRenum(ScenRoot & " " & _
                    CStr(ReNum - 1), .ChangingCells, .Comment)
                    .Delete
                End If
            End If
        End With
    Next
    End If
    
'show the next lower scenario number
    If DelNum > 1 Then ShowNum = DelNum - 1 Else ShowNum = ScenCount - 1
    With .Scenarios(ScenRoot & " " & ShowNum)
        .Show
        ScenDesc = Left(.Comment, _
        WorksheetFunction.Min(31, InStr(.Comment, ";") - 1))
    End With

'set scenario number, count and description cells
    With Sheets("Data")
        .Range("ScenThis" & ScenAbbr) = ShowNum
        .Range("ScensCnt" & ScenAbbr) = ScenCount - 1
        .Range("ScenDesc" & ScenAbbr) = ScenDesc
    End With
End With
Exit Sub
oops:
MsgBox Err & " " & Error
End Sub

'*******************************************
'* renumber scenarios after deletion       *
'*******************************************

Sub ScenRenum(ScenName As String, ScenRange As Range, ScenComment As String)
With ActiveSheet
    .Scenarios.add _
    Name:=ScenName, _
    ChangingCells:=ScenRange, _
    Comment:=ScenComment
End With
End Sub

'*******************************************
'* function returns name from abbreviation *
'*******************************************

Function Root(ScenAbbr As String) As String
Select Case ScenAbbr
    Case "Dem"
        Root = "Demand"
    Case "Sup"
        Root = "Supply"
    Case "SST"
        Root = "Safety Stock"
    Case "STI"
        Root = "Safety Time"
    Case Else
        MsgBox "Wrong name in code", vbExclamation, "Error"
End Select
End Function


Private Sub Worksheet_Activate()
Application.Calculation = xlAutomatic

End Sub

Private Sub Worksheet_Change(ByVal Target As Range)
Dim t As ListObject, r, rct, add As Integer
If Target.Address <> [horizon].Address Then Exit Sub
Application.ScreenUpdating = False
Set t = Worksheets("Demo").ListObjects("MRP")
rct = t.ListRows.Count
add = [horizon] - rct
Select Case add
  Case Is < 0
    t.ListRows([horizon] + 1).Range.Select
    If add < -1 Then Range(Selection, Selection.End(xlDown)).Select
    Selection.Delete
  Case Is = 0: Exit Sub
  Case Is > 0
    t.Resize t.Range.Resize([horizon] + 1)
    t.ListRows(9).Range.Select
    Range(Selection, Selection.End(xlDown)).Select
    Selection.FillDown
End Select
[horizon].Select
End Sub
