VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Sheet1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
Option Explicit
'***************************************
'module declarations                   *
'***************************************

'char codes for select (circles in Wingdings 2)
Const Unselected = 153, Selected = 158

'char codes for status (squares in Wingdings 2)
Const SquareDone = 162, SquareOpen = 163, SquareLo = 167, _
    SquareHi = 169, SquareCheck = 82

Const BookTitle = "Project Tracker", SheetName = "Tracker"

'Item table variables
Dim ITable As ListObject, IHRowRng, ICSelRng, ICStaRng, ICItmRng, _
    ICTxtRng, ICNamRng, ICUpdRng, ICLstRng, ICDurRng As Range
'Notes table variables
Dim NTable As ListObject, NHRowRng, NCSelRng, NCStaRng, NCItmRng, _
    NCNotRng, NCNamRng, NCUpdRng, NCStdRng, NCEndRng As Range
'other variables
Dim NewRow, TmpRng, c As Range, i, TmpRow As Integer, _
    HelpText As String, Tracker As Worksheet

'***************************************
'variable definitions                  *
'***************************************

Private Sub Definitions()
On Error GoTo oops
Set Tracker = ActiveWorkbook.Sheets("Tracker")

'Item table ranges
Set ITable = Tracker.ListObjects("Items")
With ITable
    Set IHRowRng = .HeaderRowRange
    Set ICSelRng = .ListColumns("Select").DataBodyRange
    Set ICStaRng = .ListColumns("Status").DataBodyRange
    Set ICItmRng = .ListColumns("Item").DataBodyRange
    Set ICTxtRng = .ListColumns("Text").DataBodyRange
    Set ICNamRng = .ListColumns("Name").DataBodyRange
    Set ICUpdRng = .ListColumns("Up").DataBodyRange
    Set ICLstRng = .ListColumns("Latest").DataBodyRange
    Set ICDurRng = .ListColumns("Duration").DataBodyRange
End With

'Note table ranges
Set NTable = Tracker.ListObjects("Notes")
With NTable
    Set NHRowRng = .HeaderRowRange
    Set NCSelRng = .ListColumns("Select").DataBodyRange
    Set NCStaRng = .ListColumns("Status").DataBodyRange
    Set NCItmRng = .ListColumns("Item").DataBodyRange
    Set NCNotRng = .ListColumns("Note").DataBodyRange
    Set NCNamRng = .ListColumns("Name").DataBodyRange
    Set NCUpdRng = .ListColumns("Up").DataBodyRange
    Set NCStdRng = .ListColumns("Start").DataBodyRange
    Set NCEndRng = .ListColumns("End").DataBodyRange
End With
Exit Sub

oops:
MsgBox "Valid names: Sheet = Tracker; Tables = Items, Notes", , "Invalid format"

End Sub

'***************************************
'select item subroutine                *
'called by double-click in Items       *
'***************************************
Private Sub SelectItem(Target)
If IsEmpty(Target) Then Exit Sub
Set TmpRng = Tracker.Cells(Target.Row, ICSelRng.Column)
If Asc(TmpRng) = Unselected Then
    For Each c In ICSelRng.Cells
        c.Value = Chr(Unselected)
    Next
    TmpRng.Value = Chr(Selected)
End If
Range("notes[select]").Calculate
End Sub

'***************************************
'sort table subroutine                 *
'called by double-click and right-click*
'***************************************
Private Sub SortTable(Table, Field1, Dir1, _
    Optional Field2, Optional Dir2, Optional Field3, Optional Dir3)

With Table.Sort
    .SortFields.Clear
    .SortFields.Add Key:=Field1, Order:=Dir1
    If Not IsMissing(Field2) Then .SortFields.Add Key:=Field2, Order:=Dir2
    If Not IsMissing(Field3) Then .SortFields.Add Key:=Field3, Order:=Dir3
    .Header = xlYes
    .Apply
End With
Table.DataBodyRange.Rows.EntireRow.AutoFit

End Sub

'***************************************
'split window subroutine               *
'called by double-click                *
'***************************************
Private Sub SplitWindow()

On Error Resume Next
Tracker.Activate
With ActiveWindow
    If [SplitFlag] Then
        .SplitRow = [SplitRow]
        .Panes(1).ScrollRow = 1 'IHRowRng.Row - 2
        .Panes(2).ScrollRow = NHRowRng.Row - 2
    Else
        .Split = False
        .ScrollRow = 1 'IHRowRng.Row - 3
    End If
End With

End Sub

'***************************************
'insert item subroutine                *
'called by double-click in Items       *
'***************************************
Private Sub InsertItem(Target)

With ITable.ListRows
    Set NewRow = .Add(1)
    .Item(2).Range.Copy (.Item(1).Range.Cells(1))
End With
TmpRow = NewRow.Range.Row
With Tracker
    .Cells(TmpRow, ICSelRng.Column) = Chr(Unselected)
    .Cells(TmpRow, ICItmRng.Column) = WorksheetFunction.Max(ICItmRng) + 1
    .Cells(TmpRow, ICTxtRng.Column) = ""
    .Cells(TmpRow, ICNamRng.Column) = ""
End With
Call FormatItems
Call SplitWindow
ICTxtRng(1).Select

End Sub

'***************************************
'insert note subroutine                *
'called by double-click in Items       *
'***************************************
Private Sub InsertNote(Target)

With NTable.ListRows
    Set NewRow = .Add(1)
    .Item(2).Range.Copy (.Item(1).Range.Cells(1))
End With
TmpRow = NewRow.Range.Row

With Tracker
Set TmpRng = .Cells(Target.Row, ICNamRng.Column)
    .Cells(TmpRow, NCStaRng.Column) = Chr(SquareOpen)
    .Cells(TmpRow, NCItmRng.Column) = Target
    .Cells(TmpRow, NCNotRng.Column) = ""
    .Cells(TmpRow, NCNamRng.Column) = ""
    If Within(Target, ICItmRng) Then .Cells(TmpRow, NCNamRng.Column) = TmpRng
    .Cells(TmpRow, NCStdRng.Column) = Date
    .Cells(TmpRow, NCEndRng.Column) = ""
End With
Call FormatNotes
NCNotRng(1).Select

End Sub

'***************************************
'toggle active subroutine              *
'called by double-click in Notes       *
'***************************************
Private Sub ToggleActive(Target)

With Tracker
    If Asc(Target) = SquareDone Then
        Target = Chr(SquareOpen)
        .Cells(Target.Row, NCEndRng.Column) = ""
    Else
        Target = Chr(SquareDone)
        .Cells(Target.Row, NCEndRng.Column) = Date
    End If
End With

End Sub

'***************************************
'cycle level subroutine                *
'called by right-click in Notes        *
'***************************************
Private Sub CycleLevel(Target)

Select Case Asc(Target)
    Case SquareOpen
        Target = Chr(SquareLo)
    Case SquareLo
        Target = Chr(SquareHi)
    Case SquareHi
        Target = Chr(SquareOpen)
End Select

End Sub

'***************************************
'intersection function                 *
'***************************************
Private Function Within(Target, Rng) As Boolean

If Application.Intersect(Target, Rng) Is Nothing _
Then Within = False Else Within = True

End Function

Private Sub Worksheet_Activate()
Application.Calculation = xlAutomatic

End Sub

'***************************************
'double-click actions                  *
'***************************************
Private Sub Worksheet_BeforeDoubleClick(ByVal Target As Range, Cancel As Boolean)

'*** summary of target-sensitive actions ***
' 1. if target in Items[Select] then SelectItem, SortTable, SplitWindow
' 2. if target in Items[Item] then SelectItem, InsertNote, SortTable, SplitWindow
' 3. if target in Items[Text] then InsertItem
' 4. if target in Notes[Item] then InsertNote
' 5. if target in Notes[Status] then ToggleActive
' 6. if target within tables then cancel double-click
' 7. if target is RowGrpSet then toggle RowGrp.Hidden, SplitWindow
' 8. if target is SplitSet then toggle value
' 9. if target else then revert to standard double-click action

Application.ScreenUpdating = False
Call Definitions
On Error Resume Next

target1:
'if target in Items[Select] then SelectItem, SortTable, SplitWindow
    If Not Within(Target, ICSelRng) Then GoTo target2:
    If Asc(Target) = Selected Then
        Target = Chr(Unselected)
    Else
        Call SelectItem(Target)
        Call SortTable(NTable, NCSelRng, xlDescending, NCStaRng, xlDescending, NCStdRng, xlDescending)
    End If
    [SplitSet] = Chr(SquareCheck)
    Call SplitWindow
    [RowGrp].Rows.Hidden = True
    Cancel = True
    Exit Sub

target2:
'if target in Items[Item] then SelectItem, InsertNote, SortTable, SplitWindow
    If Not Within(Target, ICItmRng) Then GoTo target3:
    If Target Then
        Call SelectItem(Target)
        Call InsertNote(Target)
        Call SortTable(ITable, ICSelRng, xlDescending, ICUpdRng, xlAscending, ICItmRng, xlAscending)
        Call SortTable(NTable, NCSelRng, xlDescending, NCStaRng, xlDescending, NCStdRng, xlDescending)
        [SplitSet] = Chr(SquareCheck)
        Call SplitWindow
        [RowGrp].Rows.Hidden = True
        ActiveWindow.Panes(2).Activate
        NCNotRng(1).Select
    End If
    Cancel = True
    Exit Sub

target3:
'if target in Items[Text] then InsertItem
    If Not Within(Target, ICTxtRng) Then GoTo target4:
    Call InsertItem(Target)
    Cancel = True
    Exit Sub

target4:
'if target in Notes[Item] then InsertNote
    If Not Within(Target, NCItmRng) Then GoTo target5:
    Call InsertNote(Target)
    NCNotRng(1).Select
    Cancel = True
    Exit Sub

target5:
'if target in Notes[Status] then ToggleActive
'ADD: if target is in Notes[Status]header then show or hide SquareDone
    If Not Within(Target, NCStaRng) Then GoTo target6:
    Call ToggleActive(Target)
    Cancel = True
    Exit Sub

target6:
'if target within tables then cancel double-click
    If Not Within(Target, ITable.Range) And _
        Not Within(Target, NTable.Range) Then GoTo target7:
    Cancel = True
    Exit Sub

target7:
'if target is RowGrpSet then toggle RowGrp.Hidden, SplitWindow
    If Within(Target, [RowGrpSet]) Then
    If [RowGrp].Rows.Hidden = True Then
        [SplitSet] = Chr(SquareOpen)
        Call SplitWindow
        [RowGrp].Rows.Hidden = False
    Else
        [SplitSet] = Chr(SquareCheck)
        Call SplitWindow
        [RowGrp].Rows.Hidden = True
    End If
    Cancel = True
    Exit Sub
    End If

target8:
'if target is SplitSet then toggle value
    With [SplitSet]
    If Target.Address = .Address Then
        If .Value = Chr(SquareOpen) Then
            .Value = Chr(SquareCheck)
        Else
            .Value = Chr(SquareOpen)
        End If
        Cancel = True
        Exit Sub
    End If
    End With

target9:
'if target else then revert to standard double-click action
    Cancel = False

Application.ScreenUpdating = True

End Sub

'***************************************
'right-click actions                   *
'***************************************
Private Sub Worksheet_BeforeRightClick(ByVal Target As Range, Cancel As Boolean)

'*** summary of target-sensitive actions ***
'1. if target in IHRowRng then SortTable Items based on column
'2. if target in NHRowRng then SortTable Notes based on column
'3. if target in NCStaRng then CycleLevel
'4. if target else then revert to standard right-click action

Application.ScreenUpdating = False
Call Definitions
On Error Resume Next

target1:
'if target in IHRowRng then SortTable Items based on column
If Not Within(Target, IHRowRng) Then GoTo target2:
Select Case Target.Column
    Case ICSelRng.Column
        Call SortTable(ITable, ICSelRng, xlDescending, ICUpdRng, xlAscending, ICItmRng, xlAscending)
    Case ICStaRng.Column
        Call SortTable(ITable, ICStaRng, xlDescending, ICUpdRng, xlAscending, ICItmRng, xlAscending)
    Case ICItmRng.Column
        Call SortTable(ITable, ICItmRng, xlAscending)
    Case ICTxtRng.Column
        Call SortTable(ITable, ICTxtRng, xlAscending, ICItmRng, xlAscending)
    Case ICNamRng.Column
        Call SortTable(ITable, ICNamRng, xlAscending, ICItmRng, xlAscending)
    Case ICUpdRng.Column
        Call SortTable(ITable, ICUpdRng, xlAscending, ICStaRng, xlDescending, ICItmRng, xlAscending)
    Case ICLstRng.Column
        Call SortTable(ITable, ICLstRng, xlDescending, ICItmRng, xlAscending)
    Case ICDurRng.Column
        Call SortTable(ITable, ICDurRng, xlDescending, ICItmRng, xlAscending)
End Select
Cancel = True
Exit Sub

target2:
'if target in NHRowRng then SortTable Notes based on column
If Not Within(Target, NHRowRng) Then GoTo target3:
Select Case Target.Column
    Case NCSelRng.Column
        Call SortTable(NTable, NCSelRng, xlDescending, NCStdRng, xlDescending, NCStaRng, xlDescending)
    Case NCStaRng.Column
        Call SortTable(NTable, NCStaRng, xlDescending, NCUpdRng, xlAscending, NCStdRng, xlDescending)
    Case NCItmRng.Column
        Call SortTable(NTable, NCItmRng, xlAscending, NCStaRng, xlDescending, NCStdRng, xlDescending)
    Case NCNotRng.Column
        Call SortTable(NTable, NCNotRng, xlAscending, NCItmRng, xlAscending)
    Case NCNamRng.Column
        Call SortTable(NTable, NCNamRng, xlAscending, NCItmRng, xlAscending)
    Case NCUpdRng.Column
        Call SortTable(NTable, NCUpdRng, xlAscending, NCEndRng, xlAscending, NCItmRng, xlAscending)
    Case NCStdRng.Column
        Call SortTable(NTable, NCStdRng, xlDescending, NCStaRng, xlDescending, NCItmRng, xlAscending)
    Case NCEndRng.Column
        Call SortTable(NTable, NCEndRng, xlAscending, NCStaRng, xlDescending, NCItmRng, xlAscending)
End Select
Cancel = True
Exit Sub

target3:
'if target in NCStaRng then CycleLevel
If Not Within(Target, NCStaRng) Then GoTo target4:
Call CycleLevel(Target)
Cancel = True
Exit Sub

target4:
'if target else then revert to standard right-click action
Cancel = False
Application.ScreenUpdating = True

End Sub

'***************************************
'calculate actions                     *
'***************************************

Private Sub Worksheet_Calculate()

'reapply SplitSet status on calculate
If ActiveWorkbook.BuiltinDocumentProperties("Title") = BookTitle _
    And ActiveSheet.Name = SheetName Then
    If ActiveWindow.Split Then
        If [SplitFlag] Then Exit Sub Else: Call SplitWindow
    Else
        If [SplitFlag] Then Call SplitWindow
    End If
End If

End Sub

'***************************************
'change actions                        *
'***************************************

Private Sub Worksheet_Change(ByVal Target As Range)
On Error GoTo out:

Call Definitions

'if target in ICItmRng then fill empty cells in ICSelRng with Chr(Unselected)
If Not Within(Target, ICItmRng) Then GoTo other:
For Each c In ICSelRng.Cells
    If IsEmpty(c.Value) Then c.Value = Chr(Unselected)
Next
GoTo out

other:
'if target in SplitRow or SplitSet then SplitWindow
If Within(Target, [SplitRow]) Or Within(Target, [SplitSet]) Then Call SplitWindow

'if target in DFSelection then SetDateFormat
If Within(Target, [DateOrder]) Or Within(Target, [DateSeparator]) Then Call SetDateFormat

out:
End Sub

'***************************************
'date format subroutine                *
'***************************************

Private Sub SetDateFormat()

With Range("Items[Latest],Notes[[Start]:[End]]")
Select Case [DateOrder]
  Case "dmy"
     Select Case [DateSeparator]
       Case "/": .NumberFormat = "dd/mm/yyyy;;"
       Case "-": .NumberFormat = "dd-mm-yyyy;;"
       Case ".": .NumberFormat = "dd.mm.yyyy;;"
     End Select
  Case "mdy"
     Select Case [DateSeparator]
       Case "/": .NumberFormat = "mm/dd/yyyy;;"
       Case "-": .NumberFormat = "mm-dd-yyyy;;"
       Case ".": .NumberFormat = "mm.dd.yyyy;;"
     End Select
  Case "ymd"
     Select Case [DateSeparator]
       Case "/": .NumberFormat = "yyyy/mm/dd;;"
       Case "-": .NumberFormat = "yyyy-mm-dd;;"
       Case ".": .NumberFormat = "yyyy.mm.dd;;"
     End Select
End Select
End With

End Sub

'***************************************
'Notes conditional formats             *
'***************************************

Private Sub FormatNotes()

'reset Notes table all columns
Range("Notes").Select
With Selection.FormatConditions
    .Delete
'IsOrphan
    With .Add(Type:=xlExpression, Formula1:="=IsOrphan")
        With .Interior
            .ThemeColor = xlThemeColorAccent2
            .TintAndShade = -0.5
        End With
        With .Font
            .ThemeColor = xlThemeColorDark1
            .TintAndShade = 0
            .Strikethrough = True
        End With
        .StopIfTrue = True
    End With
'NoteDue
    With .Add(Type:=xlExpression, Formula1:="=NoteDue")
        With .Interior
            .ThemeColor = xlThemeColorAccent2
            .TintAndShade = 0.4
        End With
        .StopIfTrue = False
    End With
'NoteToday
    With .Add(Type:=xlExpression, Formula1:="=NoteToday")
        With .Interior
            .ThemeColor = xlThemeColorAccent6
            .TintAndShade = 0.6
        End With
        .StopIfTrue = False
    End With
'NoteDone
    With .Add(Type:=xlExpression, Formula1:="=NoteDone")
        With .Interior
            .ThemeColor = xlThemeColorAccent3
            .TintAndShade = 0
        End With
        .StopIfTrue = False
    End With
'NoteSelected
    With .Add(Type:=xlExpression, Formula1:="=NoteSelected")
        With .Interior
            .ThemeColor = xlThemeColorAccent4
            .TintAndShade = 0.6
        End With
        .StopIfTrue = False
    End With
'ISFORMULA
    With .Add(Type:=xlExpression, Formula1:="=ISFORMULA(ThisCell)")
        With .Interior
            .ThemeColor = xlThemeColorAccent3
            .TintAndShade = 0.8
        End With
        .StopIfTrue = False
    End With
End With
    
'add to Item, Note, Name columns
Range("Notes[[Item]:[Name]]").Select
With Selection.FormatConditions
'LevelHi
With .Add(Type:=xlExpression, Formula1:="=LevelHi")
    With .Font
        .Bold = True
        .Color = -16776961
    End With
    .StopIfTrue = False
End With
'LevelLo
With .Add(Type:=xlExpression, Formula1:="=LevelLo")
    With .Font
        .Bold = True
        .Color = -4165632
    End With
    .StopIfTrue = False
End With
End With
    
'add to Up column
Range("Notes[Up]").Select
With Selection.FormatConditions
'hide default
    With .Add(Type:=xlExpression, Formula1:="=UpIsDefault")
        .NumberFormat = ";;;"
    End With
    .StopIfTrue = False
End With
End Sub

'***************************************
'Items conditional formats             *
'***************************************

Private Sub FormatItems()

'reset Notes table Conditional Formatting
Range("Items").Select
With Selection.FormatConditions
    .Delete
'HasNoNotes
    With .Add(Type:=xlExpression, Formula1:="=HasNoNotes")
        With .Interior
            .ThemeColor = xlThemeColorAccent2
            .TintAndShade = 0.4
        End With
        .StopIfTrue = False
    End With
'ItemToday
    With .Add(Type:=xlExpression, Formula1:="=ItemToday")
        With .Interior
            .ThemeColor = xlThemeColorAccent6
            .TintAndShade = 0.6
        End With
        .StopIfTrue = False
    End With
'ItemDone
    With .Add(Type:=xlExpression, Formula1:="=ItemDone")
        With .Interior
            .ThemeColor = xlThemeColorAccent3
            .TintAndShade = 0
        End With
        .StopIfTrue = False
    End With
'ItemSelected
    With .Add(Type:=xlExpression, Formula1:="=ItemSelected")
        With .Interior
            .ThemeColor = xlThemeColorAccent4
            .TintAndShade = 0.6
        End With
        .StopIfTrue = False
    End With
'ISFORMULA
    With .Add(Type:=xlExpression, Formula1:="=ISFORMULA(ThisCell)")
        With .Interior
            .ThemeColor = xlThemeColorAccent3
            .TintAndShade = 0.8
        End With
        .StopIfTrue = False
    End With
End With
    
End Sub

